import java.io.*; 

class EX13E
{
    public static void searchFile(String criteria, File file)
    {
        try
        {
            // Create a buffered line-number stream based on the target file.
            LineNumberInputStream linestream = new LineNumberInputStream(new BufferedInputStream(new FileInputStream(file)));
            DataInputStream dis = new DataInputStream(linestream);

            while (dis.available() > 0)
            {
                String line = dis.readLine();
                if (line.indexOf(criteria) != -1)
                {   // Found a match!
                    System.out.println("File " + file.getName() + ", line " + 
                            linestream.getLineNumber() + ":\n" + line);
                }
            }
        }
        catch (IOException e)
        {
            System.err.println("Error reading " + file.getName() + ": " + e);
        }
    }

    private static void usage()
    {
        System.err.println("Usage: grep searchtext target");
        System.err.println("Where target is a file or directory.");
    }

    public static void main(String args[])
    {
        if (args.length != 2)
        {
            System.err.println("Invalid number of arguments.");
            usage();
            return;
        }

        File target = new File(args[1]);

        if (target.isDirectory())
        {   // Retrieve a list of the files in the directory...
            String[] list = target.list();

            // ... and search each file for the search text.
            for (int i = 0; i < list.length; i++)
            {
                searchFile(args[0], new File(target, list[i]));
            }
        }
        else if (target.isFile())
        {
            // Just one file so just pass it to the search routine.
            searchFile(args[0], target);
        }
        else
        {
            System.err.println("Target is not a file or directory.");
            usage();
        }
    }
}
